@@ -1,8 +1,8 @@
 /*
- * $Id: imap.h 232 2006-11-15 23:14:41Z hubert@u.washington.edu $
+ * $Id: imap.h 380 2007-01-23 00:09:18Z hubert@u.washington.edu $
  *
  * ========================================================================
- * Copyright 2006 University of Washington
+ * Copyright 2006-2007 University of Washington
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -30,11 +30,13 @@ void    pine_sslfailure(char *, char *, unsigned long);
 #ifdef	PASSFILE
 int     get_passfile_passwd(char *, char *, char *, STRLIST_S *, int);
 int     is_using_passfile();
-void    set_passfile_passwd(char *, char *, char *, STRLIST_S *, int);
+void    set_passfile_passwd(char *, char *, char *, STRLIST_S *, int, int);
 char   *get_passfile_user(char *, STRLIST_S *);
 void    update_passfile_hostlist(char *, char *, STRLIST_S *, int);
 #ifdef	_WINDOWS
 void    erase_windows_credentials(void);
+#elif	OSX_TARGET
+void    macos_erase_keychain(void);
 #endif
 #endif
 
