@@ -23,7 +23,7 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	3 October 1995
- * Last Edited:	5 January 2007
+ * Last Edited:	29 January 2007
  */
 
 
@@ -738,7 +738,7 @@ long mbx_ping (MAILSTREAM *stream)
     if (LOCAL->filetime && (LOCAL->filetime < sbuf.st_mtime))
       LOCAL->flagcheck = T;	/* upgrade to flag checking */
 				/* new mail or flagcheck handling needed? */
-    if (((i = (sbuf.st_size - LOCAL->filesize)) || LOCAL->flagcheck ||
+    if (((sbuf.st_size - LOCAL->filesize) || LOCAL->flagcheck ||
 	 !stream->nmsgs) &&
 	((ld = lockname (lock,stream->mailbox,LOCK_EX)) >= 0)) {
       if (LOCAL->flagcheck) {	/* sweep mailbox for changed message status */
@@ -749,7 +749,8 @@ long mbx_ping (MAILSTREAM *stream)
 	  LOCAL->flagcheck =NIL;/* got all the updates */
 	}
       }
-      else if (i) ret = mbx_parse (stream);
+				/* always reparse header at least */
+      else ret = mbx_parse (stream);
       unlockfd (ld,lock);	/* release shared parse/append permission */
     }
     if (ret) {			/* must still be alive */
